@orange: #ed930d;
@green: #6a2;
@blue: #499bdd;

@fontFamily: "Open Sans", sans-serif;
@fontSize: 11px;
@fontSizeSmaller: 11px;

@mainPaneShadow: 0 1px 3px rgba(0,0,0,0.5);

@headerBackground: #222;
@headerHeight: 32px;

@footerBackground: #777;
@footerHeight: 50px;

@toolbarGradient1: #aaa;
@toolbarGradient2: darken(@toolbarGradient1, 15%);

@tableCell: #fcfcfc;
@tableBorder: #f0f0f0;

@controlShadowHighlight: ~"0 1px 1px rgba(0, 0, 0, 0.125)";
@controlShadowHighlightPressed: ~"0 1px 1px rgba(0,0,0,0.25) inset, 0 1px 1px rgba(255,255,255,0.125)";
@controlInputShadowHighlight: ~"0 1px 1px rgba(0,0,0,0.2) inset, 0 1px 1px rgba(255,255,255,0.2), 0 1px 0 rgba(0,0,0,0.04)";

@controlText: #7c8c99; //#566675;
@controlTextLighter: lighten(@controlText, 20%);
@controlTextShadowInset: 0 1px 0 rgba(255,255,255,0.25), 0 -1px 1px rgba(0,0,0,0.1);

@controlHeight: 27px;
@controlWidth: 120px;
@controlWidth2: 160px;
@controlWidth3: 240px;