body.terminal {
  background: black;
  width: 100%;
  overflow: hidden;
  text-align: center;
}
body.terminal #term {
  -moz-user-select: text;
  overflow: hidden;
  display: inline-block;
}
body.terminal #term * {
  margin: 0;
  padding: 0;
}
body.terminal #term > div {
  float: left;
  clear: both;
}
body.terminal #term pre {
  font-family: "Ubuntu Mono", monospace;
  color: white;
  font-size: 11px;
  padding: 0;
  margin: 0;
  display: inline-block;
  line-height: 17px;
  width: 5px;
}
body.terminal #term pre.cursor {
  text-decoration: underline;
}
.control.terminal-thumbnail {
  cursor: pointer;
  position: relative;
  float: left;
  padding: 5px;
}
.control.terminal-thumbnail img {
  border-radius: 5px;
  box-shadow: 0 1px 1px rgba(0, 0, 0, 0.75), 0 1px 1px rgba(255, 255, 255, 0.5) inset;
}
.control.terminal-thumbnail img:hover {
  opacity: .75;
}
.control.terminal-thumbnail a {
  position: absolute;
  top: 15px;
  right: 15px;
  background: rgba(255, 255, 255, 0.5);
  border-radius: 50%;
  width: 14px;
  height: 14px;
  text-align: center;
  padding: 3px;
  color: black;
}
.control.terminal-thumbnail a:hover {
  background: rgba(255, 255, 255, 0.75);
}
.control.terminal-thumbnail a:active {
  background: rgba(255, 255, 255, 0.25);
}
